# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/type/shared/v1/sampling.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/systemathics/apis/type/shared/v1/sampling.proto\x12 systemathics.apis.type.shared.v1*\xc0\x01\n\x08Sampling\x12\x18\n\x14SAMPLING_UNSPECIFIED\x10\x00\x12\x17\n\x13SAMPLING_ONE_MINUTE\x10\x01\x12\x18\n\x14SAMPLING_FIVE_MINUTE\x10\x02\x12\x17\n\x13SAMPLING_TEN_MINUTE\x10\x03\x12\x1b\n\x17SAMPLING_FIFTEEN_MINUTE\x10\x04\x12\x1a\n\x16SAMPLING_THIRTY_MINUTE\x10\x05\x12\x15\n\x11SAMPLING_ONE_HOUR\x10\x06\x42\xda\x01\n$com.systemathics.apis.type.shared.v1B\rSamplingProtoP\x01\xa2\x02\x04SATS\xaa\x02 Systemathics.Apis.Type.Shared.V1\xca\x02 Systemathics\\Apis\\Type\\Shared\\V1\xe2\x02,Systemathics\\Apis\\Type\\Shared\\V1\\GPBMetadata\xea\x02$Systemathics::Apis::Type::Shared::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'systemathics.apis.type.shared.v1.sampling_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n$com.systemathics.apis.type.shared.v1B\rSamplingProtoP\001\242\002\004SATS\252\002 Systemathics.Apis.Type.Shared.V1\312\002 Systemathics\\Apis\\Type\\Shared\\V1\342\002,Systemathics\\Apis\\Type\\Shared\\V1\\GPBMetadata\352\002$Systemathics::Apis::Type::Shared::V1'
  _globals['_SAMPLING']._serialized_start=86
  _globals['_SAMPLING']._serialized_end=278
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: systemathics/apis/services/daily/v1/daily_open_interest.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.type import date_pb2 as google_dot_type_dot_date__pb2
from systemathics.apis.type.shared.v1 import identifier_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_identifier__pb2
from systemathics.apis.type.shared.v1 import date_interval_pb2 as systemathics_dot_apis_dot_type_dot_shared_dot_v1_dot_date__interval__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=systemathics/apis/services/daily/v1/daily_open_interest.proto\x12#systemathics.apis.services.daily.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x16google/type/date.proto\x1a\x31systemathics/apis/type/shared/v1/identifier.proto\x1a\x34systemathics/apis/type/shared/v1/date_interval.proto\"\xbe\x01\n\x19\x44\x61ilyOpenInterestsRequest\x12L\n\nidentifier\x18\x01 \x01(\x0b\x32,.systemathics.apis.type.shared.v1.IdentifierR\nidentifier\x12S\n\rdate_interval\x18\x02 \x01(\x0b\x32..systemathics.apis.type.shared.v1.DateIntervalR\x0c\x64\x61teInterval\"h\n\x1a\x44\x61ilyOpenInterestsResponse\x12J\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x36.systemathics.apis.services.daily.v1.DailyOpenInterestR\x04\x64\x61ta\"f\n\x11\x44\x61ilyOpenInterest\x12%\n\x04\x64\x61te\x18\x01 \x01(\x0b\x32\x11.google.type.DateR\x04\x64\x61te\x12\x14\n\x05value\x18\x02 \x01(\x01R\x05value\x12\x14\n\x05score\x18\x03 \x01(\x01R\x05score2\xd4\x01\n\x19\x44\x61ilyOpenInterestsService\x12\xb6\x01\n\x12\x44\x61ilyOpenInterests\x12>.systemathics.apis.services.daily.v1.DailyOpenInterestsRequest\x1a?.systemathics.apis.services.daily.v1.DailyOpenInterestsResponse\"\x1f\x82\xd3\xe4\x93\x02\x19\x12\x17/v1/daily/openinterestsB\xf2\x01\n\'com.systemathics.apis.services.daily.v1B\x16\x44\x61ilyOpenInterestProtoP\x01\xa2\x02\x04SASD\xaa\x02#Systemathics.Apis.Services.Daily.V1\xca\x02#Systemathics\\Apis\\Services\\Daily\\V1\xe2\x02/Systemathics\\Apis\\Services\\Daily\\V1\\GPBMetadata\xea\x02\'Systemathics::Apis::Services::Daily::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'systemathics.apis.services.daily.v1.daily_open_interest_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\'com.systemathics.apis.services.daily.v1B\026DailyOpenInterestProtoP\001\242\002\004SASD\252\002#Systemathics.Apis.Services.Daily.V1\312\002#Systemathics\\Apis\\Services\\Daily\\V1\342\002/Systemathics\\Apis\\Services\\Daily\\V1\\GPBMetadata\352\002\'Systemathics::Apis::Services::Daily::V1'
  _globals['_DAILYOPENINTERESTSSERVICE'].methods_by_name['DailyOpenInterests']._options = None
  _globals['_DAILYOPENINTERESTSSERVICE'].methods_by_name['DailyOpenInterests']._serialized_options = b'\202\323\344\223\002\031\022\027/v1/daily/openinterests'
  _globals['_DAILYOPENINTERESTSREQUEST']._serialized_start=262
  _globals['_DAILYOPENINTERESTSREQUEST']._serialized_end=452
  _globals['_DAILYOPENINTERESTSRESPONSE']._serialized_start=454
  _globals['_DAILYOPENINTERESTSRESPONSE']._serialized_end=558
  _globals['_DAILYOPENINTEREST']._serialized_start=560
  _globals['_DAILYOPENINTEREST']._serialized_end=662
  _globals['_DAILYOPENINTERESTSSERVICE']._serialized_start=665
  _globals['_DAILYOPENINTERESTSSERVICE']._serialized_end=877
# @@protoc_insertion_point(module_scope)



"""Systemathics Ganymede  API Token Helpers

This module helps to create tokens to access Systemathics Ganymede authenticated API.

functions:
    * get_token - Get a JWT Authorization token suitable to call Ganymede gRPC API.
"""

import os
import http.client
import json
import pathlib
import urllib.request
import jwt
import logging

logger = logging.getLogger("token_helpers")

DEFAULT_AUDIENCE = "https://ganymede-prod"

DEFAULT_TENANT = "ganymede-prod.eu.auth0.com"

def get_token_as_metadata() -> []:
    """
    Get a JWT Authorization token suitable to call Ganymede gRPC API.
    We either use 'AUTH0_TOKEN' environment variable (if present) to create a bearer token from it.
    Or 'CLIENT_ID' and 'CLIENT_SECRET' environment variables (optionally 'AUDIENCE' can override DEFAULT_AUDIENCE, and 'TENANT' can override DEFAULT_TENANT).  
    Returns:
        A JWT Authorization token suitable to call Ganymede gRPC API in a form directly assignable to metadata= in stub call, that is [('authorization', get_token())].
    """
    return [('authorization', get_token())]

def get_token() -> str:    
    """
    Get a JWT Authorization token suitable to call Ganymede gRPC API.
    We either use 'AUTH0_TOKEN' environment variable (if present) to create a bearer token from it.
    Or 'CLIENT_ID' and 'CLIENT_SECRET' environment variables (optionally 'AUDIENCE' can override DEFAULT_AUDIENCE, and 'TENANT' can override DEFAULT_TENANT).  
    Returns:
        A JWT Authorization token suitable to call Ganymede gRPC API.
    """
    auth0_token = os.getenv("AUTH0_TOKEN","")
    client_id = os.getenv("CLIENT_ID","")
    client_secret = os.getenv("CLIENT_SECRET","")
    audience = os.getenv("AUDIENCE","")
    audience = audience if audience else DEFAULT_AUDIENCE
    tenant = os.getenv("TENANT","")
    tenant = tenant if tenant else DEFAULT_TENANT
    
    # If we have a token in AUTH0_TOKEN env var, use it as is
    if (auth0_token):
        logger.debug(f"get_token: Using token from AUTH0_TOKEN")
        return f"Bearer {auth0_token}" # valid, use it

    # If we have a persisted token
    tokendir = os.path.join(str(pathlib.Path.home()), ".systemathics")
    tokenfile = os.path.join(tokendir, _cleanup(tenant) + "_" + _cleanup(audience) + ".auth0_token.txt")
    if os.path.exists(tokenfile):
        with open(tokenfile, 'r') as input:
            auth0_token = input.read()

    if (auth0_token):
        if (_validate_token(auth0_token, tenant, audience, "from file " + tokenfile)):
            logger.debug(f"get_token: Using token from {tokenfile}")
            return f"Bearer {auth0_token}" # valid, use it
        else:
            logger.debug(f"get_token: Deleting {tokenfile} (invalid)")
            os.remove(tokenfile) # invalid, delete it
            
    # At this stage, if we don't have a valid token, ask one using Auth0 REST API (we need CLIENT_ID and CLIENT_SECRET; Optionally AUDIENCE and TENANT)
    if (client_id and client_secret):
        return _request_token_using_auth0_rest_api(
            client_id, 
            client_secret, 
            audience, 
            tenant)
    else:    
        raise Exception(f"You need to set CLIENT_ID and CLIENT_SECRET (and optionally AUDIENCE and TENANT) environment variables in order to be able to request tokens")

def _request_token_using_auth0_rest_api(client_id, client_secret, audience, tenant) -> str:
    if (client_id == ""):
        raise Exception(f"client_id cannot be null")
    if (client_secret == ""):
        raise Exception(f"client_secret cannot be null")
    if (audience == ""):
        raise Exception(f"audience cannot be null")
    if (tenant == ""):
        raise Exception(f"tenant cannot be null")

    logger.debug(f"_request_token_using_auth0_rest_api: Calling auth0 API at {tenant} to get a token")

    # Setup connection and payload
    conn = http.client.HTTPSConnection(tenant)
    headers = { "content-type": "application/json" }
    params = {"client_id": client_id, "client_secret": client_secret, "grant_type" : "client_credentials", "audience": audience }
    payload = json.dumps(params)

    # Send Request
    conn.request("POST", "/oauth/token", payload, headers)
    res = conn.getresponse()
    data = res.read()
       
    json_data =  json.loads(data.decode("utf-8"))
                            
    # Get access token to be used to authenticate against API
    try:
        token = f"{json_data['token_type']} {json_data['access_token']}"
    except Exception:
        raise Exception(f"Returned JSON doesn't contain 'token_type' and/or 'access_token'. Check your client ID, client secret, audience and tenant: {json_data}")
    
    if not _validate_token(json_data['access_token'], tenant, audience, "from AUTH0 REST API"):       
        raise Exception(f"Token returned by Auth0 API is invalid !")
                    
    # Push to file
    tokendir = os.path.join(str(pathlib.Path.home()), ".systemathics")
    tokenfile = os.path.join(tokendir, _cleanup(tenant) + "_" + _cleanup(audience) + ".auth0_token.txt")
    if not os.path.exists(tokendir):
        os.makedirs(tokendir)
    if os.path.exists(tokenfile):
        os.remove(tokenfile)
    with open(tokenfile, 'w') as output:
        output.write(json_data['access_token'])
        logger.debug(f"_request_token_using_auth0_rest_api: Pushed token to file {tokenfile}")
    return token

def _cleanup(input: str) -> str:
    str2 = input
    str2 = str2.replace('https://','')
    str2 = str2.replace('http://','')
    str2 = str2.replace('/','')
    str2 = str2.replace('\\','')
    str2 = str2.replace(':','')
    return str2
    
def _validate_token(token: str, tenant: str, audience: str, token_label: str) -> bool:
    # public key local dir/path
    pubkeydir = os.path.join(str(pathlib.Path.home()), ".systemathics")
    pubkeyfile = os.path.join(pubkeydir, _cleanup(tenant) + ".jwks.json")
    if not os.path.exists(pubkeydir):
        os.makedirs(pubkeydir)
    
    # cache
    if not os.path.exists(pubkeyfile):
        try:
            url = f'https://{tenant}/.well-known/jwks.json'
            logger.debug(f"_validate_token: Downloading public key at {url} to {pubkeyfile}")
            with urllib.request.urlopen(url) as input:
                with open(pubkeyfile, 'wb') as output:
                    pubkey = input.read()
                    output.write(pubkey)
        except urllib.error.URLError as e:
            logger.error(f"Could not get {url}: {e.reason}")
            raise
    
    jwks_url = "file:///" + pubkeyfile    
    logger.debug(f"_validate_token: Using public key store at {jwks_url}")
    jwks_client = jwt.PyJWKClient(jwks_url)
    header = jwt.get_unverified_header(token)
    kid = header["kid"]
    alg = [header["alg"]]
    key = jwks_client.get_signing_key(kid).key
    try:
        logger.debug(f"_validate_token: Validating token {token_label} with kid={kid} alg={alg} tenant={tenant} audience={audience}")
        jwt.decode(token, key, alg, audience=audience)
        logger.debug(f"_validate_token: Validated token {token_label} with kid={kid} alg={alg} tenant={tenant} audience={audience}")
        return True
    except jwt.exceptions.ExpiredSignatureError as expiredError:
        logger.error(f"_validate_token: Token is expired: {expiredError}")
        return False 
    except jwt.exceptions.DecodeError as decodeError:
        logger.error(f"_validate_token: Token could not be decoded: {decodeError}")
        return False 
    except Exception as ex:
        logger.error(f"_validate_token: Token is invalid: {ex}")
        return False   

logging.basicConfig(level=logging.DEBUG)
print(get_token())